#include <stdio.h>
#include <string.h>
#include "common_porting.h"
#include "cmsis_os.h"
#include "stm32f4xx_hal.h"
#include "bme68x_task.h"
#include "bme68x.h"


struct bme68x_dev bme68xdev;
static uint8_t dev_addr;

void bme68x_check_rslt(const char api_name[], int8_t rslt)
{
    switch (rslt)
    {
        case BME68X_OK:

            /* Do nothing */
            break;
        case BME68X_E_NULL_PTR:
            PDEBUG("API name [%s]  Error [%d] : Null pointer\r\n", api_name, rslt);
            break;
        case BME68X_E_COM_FAIL:
            PDEBUG("API name [%s]  Error [%d] : Communication failure\r\n", api_name, rslt);
            break;
        case BME68X_E_INVALID_LENGTH:
            PDEBUG("API name [%s]  Error [%d] : Incorrect length parameter\r\n", api_name, rslt);
            break;
        case BME68X_E_DEV_NOT_FOUND:
            PDEBUG("API name [%s]  Error [%d] : Device not found\r\n", api_name, rslt);
            break;
        case BME68X_E_SELF_TEST:
            PDEBUG("API name [%s]  Error [%d] : Self test error\r\n", api_name, rslt);
            break;
        case BME68X_W_NO_NEW_DATA:
            PDEBUG("API name [%s]  Warning [%d] : No new data found\r\n", api_name, rslt);
            break;
        default:
            PDEBUG("API name [%s]  Error [%d] : Unknown error code\r\n", api_name, rslt);
            break;
    }
}


int8_t BME68X_Set_ForcedMode(struct bme68x_dev *dev)
{
	int8_t rslt;
	struct bme68x_conf conf;
	struct bme68x_heatr_conf heatr_conf;

	/* Check if rslt == BME68X_OK, report or handle if otherwise */
	conf.filter = BME68X_FILTER_OFF;
	conf.odr = BME68X_ODR_NONE;
	conf.os_hum = BME68X_OS_16X;
	conf.os_pres = BME68X_OS_1X;
	conf.os_temp = BME68X_OS_2X;
	rslt = bme68x_set_conf(&conf, dev);
	bme68x_check_rslt("bme68x_set_conf", rslt);

	/* Check if rslt == BME68X_OK, report or handle if otherwise */
	heatr_conf.enable = BME68X_ENABLE;
	heatr_conf.heatr_temp = 300;
	heatr_conf.heatr_dur = 100;
	rslt = bme68x_set_heatr_conf(BME68X_FORCED_MODE, &heatr_conf, dev);
	bme68x_check_rslt("bme68x_set_heatr_conf", rslt);
	
	return rslt;
}

uint16_t sample_count =0;

int8_t BME68X_Get_Sensordata(struct bme68x_dev *dev)
{
	int8_t rslt;
	struct bme68x_conf conf;
	struct bme68x_heatr_conf heatr_conf;
	struct bme68x_data data;
	uint8_t n_fields;
	uint32_t del_period;
	uint32_t time_ms = 0;

	rslt = bme68x_set_op_mode(BME68X_FORCED_MODE, dev);
	bme68x_check_rslt("bme68x_set_op_mode", rslt);

	/* Calculate delay period in microseconds */
	del_period = bme68x_get_meas_dur(BME68X_FORCED_MODE, &conf, dev) + (heatr_conf.heatr_dur * 1000);
	dev->delay_us(del_period, dev->intf_ptr);

	time_ms = HAL_GetTick()/1000;

	/* Check if rslt == BME68X_OK, report or handle if otherwise */
	rslt = bme68x_get_data(BME68X_FORCED_MODE, &data, &n_fields, dev);
	bme68x_check_rslt("bme68x_get_data", rslt);

	sample_count++;

#ifdef BME68X_USE_FPU
	PDEBUG("%u, %lu, T:%.2f degC, P:%.2f hPa, H:%.2f %%rH, %.2f, 0x%x\n",
	   sample_count,
	   (long unsigned int)time_ms,
	   data.temperature,
	   data.pressure,
	   data.humidity,
	   data.gas_resistance,
	   data.status);
#else
	PDEBUG("%u, %lu, %d, %lu, %lu, %lu, 0x%x\n",
	   sample_count,
	   (long unsigned int)time_ms,
	   (data.temperature / 100),
	   (long unsigned int)data.pressure,
	   (long unsigned int)(data.humidity / 1000),
	   (long unsigned int)data.gas_resistance,
	   data.status);
#endif
#if 0
	PDEBUG("T: %.2f degC, P: %.2f hPa, H %.2f %%rH , meas_period=%d", data.temperature / 100.0f, data.pressure / 100.0f, data.humidity / 1000.0f, meas_period);
	/* Avoid using measurements from an unstable heating setup */
	if(data.status & BME680_GASM_VALID_MSK)
	{
		PDEBUG(", G: %d ohms", data.gas_resistance);
	}

#endif

	return rslt;
}

int8_t Init_BME68X(struct bme68x_dev *dev)
{
	int8_t rslt = BME68X_OK;
	
#if defined(USE_I2C_INTERFACE)
	PDEBUG("I2C Interface\n");
	dev_addr = BME68X_I2C_ADDR_HIGH;
	dev->intf = BME68X_I2C_INTF;
	dev->read = SensorAPI_I2Cx_Read;
	dev->write = SensorAPI_I2Cx_Write;
#elif defined(USE_SPI_INTERFACE)
	PDEBUG("SPI Interface\n");
	dev->intf = BME68X_SPI_INTF;
	dev->read = SensorAPI_SPIx_Read;
	dev->write = SensorAPI_SPIx_Write;
#endif
	dev->delay_us = bmeb8x_delay_us;
	dev->intf_ptr = &dev_addr;
	dev->amb_temp = 25; /* The ambient temperature in deg C is used for defining the heater temperature */

	bme68x_soft_reset(dev);
	dev->delay_us(10, dev->intf_ptr);
	
	rslt = bme68x_init(dev);
	if(rslt != BME68X_OK)
	{
		PDEBUG("bme68x_init failed, rslt=%d\r\n", rslt);
	}
	else
	{
		PDEBUG("bme68x_init successful, chip id=0x%02X\r\n", dev->chip_id);
	}

	rslt = BME68X_Set_ForcedMode(dev);
	if(rslt != BME68X_OK)
	{
		PDEBUG("BME68X_Set_ForcedMode failed, rslt=%d\r\n", rslt);
	}
	
	return rslt;
}

void StartBME68XTask(void const * argument)
{
	int8_t rslt = BME68X_OK;
	struct bme68x_dev *dev;

	dev = &bme68xdev;

	Init_BME68X(dev);

	for(;;)
	{
		BME68X_Get_Sensordata(dev);
		vTaskDelay(1000);
	}	
}


